@@ -3,8 +3,8 @@
     sphinx.directives.desc
     ~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: 2007-2008 by Georg Brandl.
-    :license: BSD.
+    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
 import re
@@ -14,23 +14,24 @@ from docutils import nodes
 from docutils.parsers.rst import directives
 
 from sphinx import addnodes
+from sphinx.util import ws_re
 
 
-ws_re = re.compile(r'\s+')
-
 # ------ information units ---------------------------------------------------------
 
-def desc_index_text(desctype, module, name):
+def desc_index_text(desctype, module, name, add_modules):
     if desctype == 'function':
         if not module:
-            return '%s() (built-in function)' % name
-        return '%s() (in module %s)' % (name, module)
+            return _('%s() (built-in function)') % name
+        return _('%s() (in module %s)') % (name, module)
     elif desctype == 'data':
         if not module:
-            return '%s (built-in variable)' % name
-        return '%s (in module %s)' % (name, module)
+            return _('%s (built-in variable)') % name
+        return _('%s (in module %s)') % (name, module)
     elif desctype == 'class':
-        return '%s (class in %s)' % (name, module)
+        if not module:
+            return _('%s (built-in class)') % name
+        return _('%s (class in %s)') % (name, module)
     elif desctype == 'exception':
         return name
     elif desctype == 'method':
@@ -38,82 +39,100 @@ def desc_index_text(desctype, module, name):
             clsname, methname = name.rsplit('.', 1)
         except ValueError:
             if module:
-                return '%s() (in module %s)' % (name, module)
+                return _('%s() (in module %s)') % (name, module)
             else:
                 return '%s()' % name
-        if module:
-            return '%s() (%s.%s method)' % (methname, module, clsname)
+        if module and add_modules:
+            return _('%s() (%s.%s method)') % (methname, module, clsname)
         else:
-            return '%s() (%s method)' % (methname, clsname)
+            return _('%s() (%s method)') % (methname, clsname)
     elif desctype == 'staticmethod':
         try:
             clsname, methname = name.rsplit('.', 1)
         except ValueError:
             if module:
-                return '%s() (in module %s)' % (name, module)
+                return _('%s() (in module %s)') % (name, module)
             else:
                 return '%s()' % name
-        if module:
-            return '%s() (%s.%s static method)' % (methname, module, clsname)
+        if module and add_modules:
+            return _('%s() (%s.%s static method)') % (methname, module, clsname)
         else:
-            return '%s() (%s static method)' % (methname, clsname)
+            return _('%s() (%s static method)') % (methname, clsname)
     elif desctype == 'attribute':
         try:
             clsname, attrname = name.rsplit('.', 1)
         except ValueError:
             if module:
-                return '%s (in module %s)' % (name, module)
+                return _('%s (in module %s)') % (name, module)
             else:
                 return name
-        if module:
-            return '%s (%s.%s attribute)' % (attrname, module, clsname)
+        if module and add_modules:
+            return _('%s (%s.%s attribute)') % (attrname, module, clsname)
         else:
-            return '%s (%s attribute)' % (attrname, clsname)
-    elif desctype == 'opcode':
-        return '%s (opcode)' % name
+            return _('%s (%s attribute)') % (attrname, clsname)
     elif desctype == 'cfunction':
-        return '%s (C function)' % name
+        return _('%s (C function)') % name
     elif desctype == 'cmember':
-        return '%s (C member)' % name
+        return _('%s (C member)') % name
     elif desctype == 'cmacro':
-        return '%s (C macro)' % name
+        return _('%s (C macro)') % name
     elif desctype == 'ctype':
-        return '%s (C type)' % name
+        return _('%s (C type)') % name
     elif desctype == 'cvar':
-        return '%s (C variable)' % name
+        return _('%s (C variable)') % name
     else:
-        raise ValueError("unhandled descenv: %s" % desctype)
+        raise ValueError('unhandled descenv: %s' % desctype)
 
 
 # ------ make field lists (like :param foo:) in desc bodies prettier
 
+_ = lambda x: x  # make gettext extraction in constants possible
+
 doc_fields_with_arg = {
-    'param': 'param',
-    'parameter': 'param',
-    'arg': 'param',
-    'argument': 'param',
-    'keyword': 'param',
-    'kwarg': 'param',
-    'kwparam': 'param',
-    'type': 'type',
-    'raises': 'Raises',
+    'param': '%param',
+    'parameter': '%param',
+    'arg': '%param',
+    'argument': '%param',
+    'keyword': '%param',
+    'kwarg': '%param',
+    'kwparam': '%param',
+    'type': '%type',
+    'raises': _('Raises'),
     'raise': 'Raises',
     'exception': 'Raises',
     'except': 'Raises',
-    'var': 'Variable',
+    'var': _('Variable'),
     'ivar': 'Variable',
     'cvar': 'Variable',
-    'returns': 'Returns',
+    'returns': _('Returns'),
     'return': 'Returns',
 }
 
+doc_fields_with_linked_arg = ('raises', 'raise', 'exception', 'except')
+
 doc_fields_without_arg = {
     'returns': 'Returns',
     'return': 'Returns',
-    'rtype': 'Return type',
+    'rtype': _('Return type'),
 }
 
-def handle_doc_fields(node):
+del _
+
+
+def _is_only_paragraph(node):
+    # determine if the node only contains one paragraph (and system messages)
+    if len(node) == 0:
+        return False
+    elif len(node) > 1:
+        for subnode in node[1:]:
+            if not isinstance(subnode, nodes.system_message):
+                return False
+    if isinstance(node[0], nodes.paragraph):
+        return True
+    return False
+
+
+def handle_doc_fields(node, env):
     # don't traverse, only handle field lists that are immediate children
     for child in node.children:
         if not isinstance(child, nodes.field_list):
@@ -126,16 +145,15 @@ def handle_doc_fields(node):
             fname, fbody = field
             try:
                 typ, obj = fname.astext().split(None, 1)
-                typ = doc_fields_with_arg[typ]
-                if len(fbody.children) == 1 and \
-                   isinstance(fbody.children[0], nodes.paragraph):
+                typdesc = _(doc_fields_with_arg[typ])
+                if _is_only_paragraph(fbody):
                     children = fbody.children[0].children
                 else:
                     children = fbody.children
-                if typ == 'param':
+                if typdesc == '%param':
                     if not params:
                         pfield = nodes.field()
-                        pfield += nodes.field_name('Parameters', 'Parameters')
+                        pfield += nodes.field_name('', _('Parameters'))
                         pfield += nodes.field_body()
                         params = nodes.bullet_list()
                         pfield[1] += params
@@ -148,19 +166,33 @@ def handle_doc_fields(node):
                     param_nodes[obj] = dlpar
                     dlitem += dlpar
                     params += dlitem
-                elif typ == 'type':
-                    param_types[obj] = fbody.astext()
+                elif typdesc == '%type':
+                    typenodes = fbody.children
+                    if _is_only_paragraph(fbody):
+                        typenodes = [nodes.Text(' (')] + \
+                                    typenodes[0].children + [nodes.Text(')')]
+                    param_types[obj] = typenodes
                 else:
-                    fieldname = typ + ' ' + obj
+                    fieldname = typdesc + ' '
                     nfield = nodes.field()
-                    nfield += nodes.field_name(fieldname, fieldname)
+                    nfieldname = nodes.field_name(fieldname, fieldname)
+                    nfield += nfieldname
+                    node = nfieldname
+                    if typ in doc_fields_with_linked_arg:
+                        node = addnodes.pending_xref(obj, reftype='obj',
+                                                     refcaption=False,
+                                                     reftarget=obj,
+                                                     modname=env.currmodule,
+                                                     classname=env.currclass)
+                        nfieldname += node
+                    node += nodes.Text(obj, obj)
                     nfield += nodes.field_body()
                     nfield[1] += fbody.children
                     new_list += nfield
             except (KeyError, ValueError):
                 fnametext = fname.astext()
                 try:
-                    typ = doc_fields_without_arg[fnametext]
+                    typ = _(doc_fields_without_arg[fnametext])
                 except KeyError:
                     # at least capitalize the field name
                     typ = fnametext.capitalize()
@@ -168,7 +200,7 @@ def handle_doc_fields(node):
                 new_list += field
         for param, type in param_types.iteritems():
             if param in param_nodes:
-                param_nodes[param].insert(1, nodes.Text(' (%s)' % type))
+                param_nodes[param][1:1] = type
         child.replace_self(new_list)
 
 
@@ -197,6 +229,9 @@ def parse_py_signature(signode, sig, desctype, module, env):
         raise ValueError
     classname, name, arglist, retann = m.groups()
 
+    if retann:
+        retann = u' \N{RIGHTWARDS ARROW} ' + retann.strip()[2:]
+
     if env.currclass:
         add_module = False
         if classname and classname.startswith(env.currclass):
@@ -231,6 +266,8 @@ def parse_py_signature(signode, sig, desctype, module, env):
         if desctype in ('function', 'method', 'staticmethod'):
             # for callables, add an empty parameter list
             signode += addnodes.desc_parameterlist()
+        if retann:
+            signode += addnodes.desc_type(retann, retann)
         return fullname, classname
     signode += addnodes.desc_parameterlist()
 
@@ -253,7 +290,6 @@ def parse_py_signature(signode, sig, desctype, module, env):
     if len(stack) != 1:
         raise ValueError
     if retann:
-        retann = u' \N{RIGHTWARDS ARROW} ' + retann.strip()[2:]
         signode += addnodes.desc_type(retann, retann)
     return fullname, classname
 
@@ -283,7 +319,7 @@ def parse_c_type(node, ctype):
     # add cross-ref nodes for all words
     for part in filter(None, wsplit_re.split(ctype)):
         tnode = nodes.Text(part, part)
-        if part[0] in string.letters+'_' and part not in stopwords:
+        if part[0] in string.ascii_letters+'_' and part not in stopwords:
             pnode = addnodes.pending_xref(
                 '', reftype='ctype', reftarget=part, modname=None, classname=None)
             pnode += tnode
@@ -301,7 +337,7 @@ def parse_c_signature(signode, sig, desctype):
         raise ValueError('no match')
     rettype, name, arglist, const = m.groups()
 
-    signode += addnodes.desc_type("", "")
+    signode += addnodes.desc_type('', '')
     parse_c_type(signode[-1], rettype)
     try:
         classname, funcname = name.split('::', 1)
@@ -343,33 +379,18 @@ def parse_c_signature(signode, sig, desctype):
     return name
 
 
-opcode_sig_re = re.compile(r'(\w+(?:\+\d)?)\s*\((.*)\)')
-
-def parse_opcode_signature(signode, sig):
-    """Transform an opcode signature into RST nodes."""
-    m = opcode_sig_re.match(sig)
-    if m is None:
-        raise ValueError
-    opname, arglist = m.groups()
-    signode += addnodes.desc_name(opname, opname)
-    paramlist = addnodes.desc_parameterlist()
-    signode += paramlist
-    paramlist += addnodes.desc_parameter(arglist, arglist)
-    return opname.strip()
-
-
 option_desc_re = re.compile(
-    r'(/|-|--)([-_a-zA-Z0-9]+)(\s*.*?)(?=,\s+(?:/|-|--)|$)')
+    r'((?:/|-|--)[-_a-zA-Z0-9]+)(\s*.*?)(?=,\s+(?:/|-|--)|$)')
 
 def parse_option_desc(signode, sig):
     """Transform an option description into RST nodes."""
     count = 0
     firstname = ''
     for m in option_desc_re.finditer(sig):
-        prefix, optname, args = m.groups()
+        optname, args = m.groups()
         if count:
             signode += addnodes.desc_addname(', ', ', ')
-        signode += addnodes.desc_name(prefix+optname, prefix+optname)
+        signode += addnodes.desc_name(optname, optname)
         signode += addnodes.desc_addname(args, args)
         if not count:
             firstname = optname
@@ -379,16 +400,20 @@ def parse_option_desc(signode, sig):
     return firstname
 
 
+strip_backslash_re = re.compile(r'\\(?=[^\\])')
+
 def desc_directive(desctype, arguments, options, content, lineno,
                    content_offset, block_text, state, state_machine):
     env = state.document.settings.env
+    inode = addnodes.index(entries=[])
     node = addnodes.desc()
     node['desctype'] = desctype
 
     noindex = ('noindex' in options)
     node['noindex'] = noindex
     # remove backslashes to support (dummy) escapes; helps Vim's highlighting
-    signatures = map(lambda s: s.strip().replace('\\', ''), arguments[0].split('\n'))
+    signatures = map(lambda s: strip_backslash_re.sub('', s.strip()),
+                     arguments[0].split('\n'))
     names = []
     clsname = None
     module = options.get('module', env.currmodule)
@@ -405,17 +430,20 @@ def desc_directive(desctype, arguments, options, content, lineno,
                 name, clsname = parse_py_signature(signode, sig, desctype, module, env)
             elif desctype in ('cfunction', 'cmember', 'cmacro', 'ctype', 'cvar'):
                 name = parse_c_signature(signode, sig, desctype)
-            elif desctype == 'opcode':
-                name = parse_opcode_signature(signode, sig)
             elif desctype == 'cmdoption':
                 optname = parse_option_desc(signode, sig)
                 if not noindex:
-                    targetname = 'cmdoption-' + optname
+                    targetname = optname.replace('/', '-')
+                    if env.currprogram:
+                        targetname = '-' + env.currprogram + targetname
+                    targetname = 'cmdoption' + targetname
                     signode['ids'].append(targetname)
                     state.document.note_explicit_target(signode)
-                    env.note_index_entry('pair', 'command line option; %s' % sig,
-                                         targetname, targetname)
-                    env.note_reftarget('option', optname, targetname)
+                    inode['entries'].append(
+                        ('pair', _('%scommand line option; %s') %
+                         ((env.currprogram and env.currprogram + ' ' or ''), sig),
+                         targetname, targetname))
+                    env.note_progoption(optname, targetname)
                 continue
             elif desctype == 'describe':
                 signode.clear()
@@ -436,14 +464,14 @@ def desc_directive(desctype, arguments, options, content, lineno,
                     signode['ids'].append(targetname)
                     state.document.note_explicit_target(signode)
                     if indextemplate:
-                        indexentry = indextemplate % (fullname,)
+                        indexentry = _(indextemplate) % (fullname,)
                         indextype = 'single'
                         colon = indexentry.find(':')
                         if colon != -1:
                             indextype = indexentry[:colon].strip()
                             indexentry = indexentry[colon+1:].strip()
-                        env.note_index_entry(indextype, indexentry,
-                                             targetname, targetname)
+                        inode['entries'].append((indextype, indexentry,
+                                                 targetname, targetname))
                     env.note_reftarget(rolename, fullname, targetname)
                 # don't use object indexing below
                 continue
@@ -465,9 +493,9 @@ def desc_directive(desctype, arguments, options, content, lineno,
                 env.note_descref(fullname, desctype, lineno)
             names.append(name)
 
-            env.note_index_entry('single',
-                                 desc_index_text(desctype, module, name),
-                                 fullname, fullname)
+            indextext = desc_index_text(desctype, module, name,
+                                        env.config.add_module_names)
+            inode['entries'].append(('single', indextext, fullname, fullname))
 
     subnode = addnodes.desc_content()
     # needed for automatic qualification of members
@@ -483,12 +511,12 @@ def desc_directive(desctype, arguments, options, content, lineno,
     if names:
         env.currdesc = names[0]
     state.nested_parse(content, content_offset, subnode)
-    handle_doc_fields(subnode)
+    handle_doc_fields(subnode, env)
     if clsname_set:
         env.currclass = None
     env.currdesc = None
     node.append(subnode)
-    return [node]
+    return [inode, node]
 
 desc_directive.content = 1
 desc_directive.arguments = (1, 0, 1)
@@ -510,8 +538,6 @@ desctypes = [
     'cmacro',
     'ctype',
     'cvar',
-    # the odd one
-    'opcode',
     # for command line options
     'cmdoption',
     # the generic one
@@ -522,13 +548,17 @@ desctypes = [
 for _name in desctypes:
     directives.register_directive(_name, desc_directive)
 
+_ = lambda x: x
+
 # Generic cross-reference types; they can be registered in the application;
 # the directives are either desc_directive or target_directive
 additional_xref_types = {
     # directive name: (role name, index text, function to parse the desc node)
-    'envvar': ('envvar', 'environment variable; %s', None),
+    'envvar': ('envvar', _('environment variable; %s'), None),
 }
 
+del _
+
 
 # ------ target --------------------------------------------------------------------
 
@@ -536,12 +566,13 @@ def target_directive(targettype, arguments, options, content, lineno,
                      content_offset, block_text, state, state_machine):
     """Generic target for user-defined cross-reference types."""
     env = state.document.settings.env
-    rolename, indextemplate, _ = additional_xref_types[targettype]
+    rolename, indextemplate, foo = additional_xref_types[targettype]
     # normalize whitespace in fullname like xfileref_role does
     fullname = ws_re.sub('', arguments[0].strip())
     targetname = '%s-%s' % (rolename, fullname)
     node = nodes.target('', '', ids=[targetname])
     state.document.note_explicit_target(node)
+    ret = [node]
     if indextemplate:
         indexentry = indextemplate % (fullname,)
         indextype = 'single'
@@ -549,9 +580,10 @@ def target_directive(targettype, arguments, options, content, lineno,
         if colon != -1:
             indextype = indexentry[:colon].strip()
             indexentry = indexentry[colon+1:].strip()
-        env.note_index_entry(indextype, indexentry, targetname, targetname)
+        inode = addnodes.index(entries=[(indextype, indexentry, targetname, targetname)])
+        ret.insert(0, inode)
     env.note_reftarget(rolename, fullname, targetname)
-    return [node]
+    return ret
 
 target_directive.content = 0
 target_directive.arguments = (1, 0, 1)
