@@ -3,8 +3,8 @@
     sphinx.directives.other
     ~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: 2007-2008 by Georg Brandl.
-    :license: BSD.
+    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
 import re
@@ -14,8 +14,8 @@ from docutils import nodes
 from docutils.parsers.rst import directives
 
 from sphinx import addnodes
-from sphinx.util import patfilter
-from sphinx.roles import caption_ref_re
+from sphinx.locale import pairindextypes
+from sphinx.util import patfilter, ws_re, caption_ref_re
 from sphinx.util.compat import make_admonition
 
 
@@ -98,13 +98,15 @@ def module_directive(name, arguments, options, content, lineno,
     if 'platform' in options:
         modulenode['platform'] = options['platform']
         node = nodes.paragraph()
-        node += nodes.emphasis('Platforms: ', 'Platforms: ')
+        node += nodes.emphasis('', _('Platforms: '))
         node += nodes.Text(options['platform'], options['platform'])
         ret.append(node)
     # the synopsis isn't printed; in fact, it is only used in the modindex currently
     if not noindex:
-        env.note_index_entry('single', '%s (module)' % modname,
-                             'module-' + modname, modname)
+        indextext = _('%s (module)') % modname
+        inode = addnodes.index(entries=[('single', indextext,
+                                         'module-' + modname, modname)])
+        ret.insert(0, inode)
     return ret
 
 module_directive.arguments = (1, 0, 0)
@@ -141,11 +143,11 @@ def author_directive(name, arguments, options, content, lineno,
     emph = nodes.emphasis()
     para += emph
     if name == 'sectionauthor':
-        text = 'Section author: '
+        text = _('Section author: ')
     elif name == 'moduleauthor':
-        text = 'Module author: '
+        text = _('Module author: ')
     else:
-        text = 'Author: '
+        text = _('Author: ')
     emph += nodes.Text(text, text)
     inodes, messages = state.inline_text(arguments[0], lineno)
     emph.extend(inodes)
@@ -156,11 +158,24 @@ directives.register_directive('sectionauthor', author_directive)
 directives.register_directive('moduleauthor', author_directive)
 
 
+def program_directive(name, arguments, options, content, lineno,
+                      content_offset, block_text, state, state_machine):
+    env = state.document.settings.env
+    program = ws_re.sub('-', arguments[0].strip())
+    if program == 'None':
+        env.currprogram = None
+    else:
+        env.currprogram = program
+    return []
+
+program_directive.arguments = (1, 0, 1)
+directives.register_directive('program', program_directive)
+
+
 # ------ index markup --------------------------------------------------------------
 
-entrytypes = [
-    'single', 'pair', 'triple', 'module', 'keyword', 'operator',
-    'object', 'exception', 'statement', 'builtin',
+indextypes = [
+    'single', 'pair', 'triple',
 ]
 
 def index_directive(name, arguments, options, content, lineno,
@@ -175,17 +190,25 @@ def index_directive(name, arguments, options, content, lineno,
     indexnode['entries'] = ne = []
     for entry in arguments:
         entry = entry.strip()
-        for type in entrytypes:
+        for type in pairindextypes:
             if entry.startswith(type+':'):
                 value = entry[len(type)+1:].strip()
-                env.note_index_entry(type, value, targetid, value)
-                ne.append((type, value, targetid, value))
+                value = pairindextypes[type] + '; ' + value
+                ne.append(('pair', value, targetid, value))
                 break
-        # shorthand notation for single entries
         else:
-            for value in entry.split(','):
-                env.note_index_entry('single', value.strip(), targetid, value.strip())
-                ne.append(('single', value.strip(), targetid, value.strip()))
+            for type in indextypes:
+                if entry.startswith(type+':'):
+                    value = entry[len(type)+1:].strip()
+                    ne.append((type, value, targetid, value))
+                    break
+            # shorthand notation for single entries
+            else:
+                for value in entry.split(','):
+                    value = value.strip()
+                    if not value:
+                        continue
+                    ne.append(('single', value, targetid, value))
     return [indexnode, targetnode]
 
 index_directive.arguments = (1, 0, 1)
@@ -222,13 +245,19 @@ directives.register_directive('versionchanged', version_directive)
 
 def seealso_directive(name, arguments, options, content, lineno,
                       content_offset, block_text, state, state_machine):
-    rv = make_admonition(
-        addnodes.seealso, name, ['See also'], options, content,
+    ret = make_admonition(
+        addnodes.seealso, name, [_('See also')], options, content,
         lineno, content_offset, block_text, state, state_machine)
-    return rv
+    if arguments:
+        argnodes, msgs = state.inline_text(arguments[0], lineno)
+        para = nodes.paragraph()
+        para += argnodes
+        para += msgs
+        ret[0].insert(1, para)
+    return ret
 
 seealso_directive.content = 1
-seealso_directive.arguments = (0, 0, 0)
+seealso_directive.arguments = (0, 1, 1)
 directives.register_directive('seealso', seealso_directive)
 
 
@@ -292,7 +321,7 @@ directives.register_directive('productionlist', productionlist_directive)
 
 def glossary_directive(name, arguments, options, content, lineno,
                        content_offset, block_text, state, state_machine):
-    """Glossary with cross-reference targets for :dfn: roles."""
+    """Glossary with cross-reference targets for :term: roles."""
     env = state.document.settings.env
     node = addnodes.glossary()
     state.nested_parse(content, content_offset, node)
@@ -314,6 +343,10 @@ def glossary_directive(name, arguments, options, content, lineno,
             li[0]['ids'].append(new_id)
             state.document.settings.env.note_reftarget('term', termtext.lower(),
                                                        new_id)
+            # add an index entry too
+            indexnode = addnodes.index()
+            indexnode['entries'] = [('single', termtext, new_id, termtext)]
+            li.insert(0, indexnode)
     return [node]
 
 glossary_directive.content = 1
