@@ -23,7 +23,9 @@ sys.path.append(os.path.abspath('.'))
 
 # Add any Sphinx extension module names here, as strings. They can be extensions
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
-extensions = ['ext']
+extensions = ['ext', 'sphinx.ext.autodoc', 'sphinx.ext.jsmath',
+              'sphinx.ext.coverage', 'sphinx.ext.todo']
+jsmath_path = 'dummy.js'
 
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
@@ -32,11 +34,11 @@ templates_path = ['_templates']
 source_suffix = '.txt'
 
 # The master toctree document.
-master_doc = 'index'
+master_doc = 'contents'
 
 # General substitutions.
-project = 'Sphinx Tests'
-copyright = '2008, Georg Brandl'
+project = 'Sphinx <Tests>'
+copyright = '2008, Georg Brandl & Team'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
@@ -57,7 +59,9 @@ today_fmt = '%B %d, %Y'
 
 # List of directories, relative to source directories, that shouldn't be searched
 # for source files.
-#exclude_dirs = []
+exclude_trees = ['_build']
+
+keep_warnings = True
 
 # If true, '()' will be appended to :func: etc. cross-reference text.
 #add_function_parentheses = True
@@ -130,6 +134,8 @@ html_last_updated_fmt = '%b %d, %Y'
 # If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
 #html_file_suffix = ''
 
+html_context = {'hckey': 'hcval'}
+
 # Output file base name for HTML help builder.
 htmlhelp_basename = 'SphinxTestsdoc'
 
@@ -146,7 +152,8 @@ htmlhelp_basename = 'SphinxTestsdoc'
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, document class [howto/manual]).
 latex_documents = [
-  ('contents', 'SphinxTests.tex', 'Sphinx Tests Documentation', 'Georg Brandl', 'manual'),
+  ('contents', 'SphinxTests.tex', 'Sphinx Tests Documentation',
+   'Georg Brandl', 'manual'),
 ]
 
 # The name of an image file (relative to this directory) to place at the top of
@@ -168,6 +175,9 @@ latex_documents = [
 
 value_from_conf_py = 84
 
+coverage_c_path = ['special/*.h']
+coverage_c_regexes = {'cfunction': r'^PyAPI_FUNC\(.*\)\s+([^_][\w_]+)'}
+
 
 def setup(app):
     app.add_config_value('value_from_conf_py', 42, False)
