@@ -6,8 +6,8 @@
     Test the sphinx.config.Config class and its handling in the
     Application class.
 
-    :copyright: 2008 by Georg Brandl.
-    :license: BSD.
+    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
 from util import *
@@ -15,13 +15,13 @@ from util import *
 from sphinx.application import ExtensionError
 
 
-def test_core_config():
-    overrides = {'master_doc': 'master', 'nonexisting_value': 'True'}
-    cfg = TestApp(confoverrides=overrides).config
+@with_app(confoverrides={'master_doc': 'master', 'nonexisting_value': 'True'})
+def test_core_config(app):
+    cfg = app.config
 
     # simple values
     assert 'project' in cfg.__dict__
-    assert cfg.project == 'Sphinx Tests'
+    assert cfg.project == 'Sphinx <Tests>'
     assert cfg.templates_path == ['_templates']
 
     # overrides
@@ -34,7 +34,7 @@ def test_core_config():
 
     # complex default values
     assert 'html_title' not in cfg.__dict__
-    assert cfg.html_title == 'Sphinx Tests v0.4alpha1 documentation'
+    assert cfg.html_title == 'Sphinx <Tests> v0.4alpha1 documentation'
 
     # complex default values mustn't raise
     for valuename in cfg.config_values:
@@ -61,8 +61,8 @@ def test_core_config():
     assert cfg['project'] == cfg.project == 'Sphinx Tests'
 
 
-def test_extension_values():
-    app = TestApp()
+@with_app()
+def test_extension_values(app):
     cfg = app.config
 
     # default value
